/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 8 -*-
 * vim:expandtab:shiftwidth=8:tabstop=8:
 *
 * Copyright (C) 2001, 2002 Cluster File Systems, Inc.
 *   Author: Peter J. Braam <braam@clusterfs.com>
 *   Author: Phil Schwan <phil@clusterfs.com>
 *
 *   This file is part of InterMezzo, http://www.inter-mezzo.org.
 *
 *   InterMezzo is free software; you can redistribute it and/or
 *   modify it under the terms of version 2 of the GNU General Public
 *   License as published by the Free Software Foundation.
 *
 *   InterMezzo is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with InterMezzo; if not, write to the Free Software
 *
 */

#ifndef __INTERSYNC_IZOLIB_H
#define __INTERSYNC_IZOLIB_H

#include <asm/types.h>
#include <uuid/uuid.h>
#include <linux/intermezzo_fs.h>

#ifndef __KERNEL__
#define ATTR_MODE       1
#define ATTR_UID        2
#define ATTR_GID        4
#define ATTR_SIZE       8
#define ATTR_ATIME      16
#define ATTR_MTIME      32
#define ATTR_CTIME      64
#define ATTR_ATIME_SET  128
#define ATTR_MTIME_SET  256
#define ATTR_FORCE      512     /* Not a change, but a change it */
#define ATTR_ATTR_FLAG  1024
#endif

/* httpclient.c */
struct httpclient {
        unsigned char *body;
        size_t len;
        void *_handle;  /* don't touch */
};

int httpclient_fetch(struct httpclient *client, const char *uri,
                size_t start_byte, size_t end_byte);
int httpclient_fetch_file(struct httpclient *client, const char *uri,
                size_t start_byte, size_t end_byte, FILE *fp);
static inline struct httpclient *httpclient_alloc(void)
{
        return (struct httpclient *) g_malloc0(sizeof(struct httpclient));
}
void httpclient_reset(struct httpclient *client);
void httpclient_free(struct httpclient *client);
const char *httpclient_strerror(int rc);
int httpclient_errno(int rc);

/* cgi.c */
const char *cgi_getenv(const char *var);
int cgi_getenv_u64(const char *var, __u64 *ret);
#define CGI_ERROR(fmt, a...)                 \
do {                                            \
        char * str = getenv("REMOTE_ADDR");    \
        fprintf(stderr, "%s: " fmt, str , ## a);     \
} while (0)


/* fset.c */
int fset_cgi_open(const char * docroot, const char *fsetname);
int cache_cgi_open(const char * docroot);

/* base64.c */
void izo_to_base64 (unsigned char *out, size_t olen, const unsigned char *in, 
                    size_t len);
int izo_from_base64 (char *out, const char *in);


/* config.c */
struct is_config {
        char *cfg_psdev;
        char *cfg_uuid;

        char *cfg_docroot;
        char *cfg_cache;
        char *cfg_server;
        int   cfg_server_port;
        char *cfg_local_ip;
        int   cfg_local_port;
        char *cfg_hoard_conf;
        int   cfg_reint_only;
        int   cfg_daemon;
        char *cfg_clientid;
        char *cfg_fsetname;
};

struct is_config *is_parse_response(char * rsp);
struct is_config *read_local_config(char *cache);
int write_local_config(struct is_config *cfg);
void config_apache_conf(char *configfile, char *ip, int port, char *docroot,
                        char *cgi_dir);
int parse_configuration(struct is_config *is_cfg, char *file);

/* ioctl.c */
int izo_ioc_mark(int mark_what, int fd, int and_flags, int or_flags);
int izo_ioc_mark_fset(int fd, int and_flags, int or_flags);
int izo_ioc_clear_all_fsets(int fd);
int izo_ioc_clear_fset(int fd);
int izo_ioc_set_fset(int fd, char *fset_name, int flags);
int izo_ioc_get_flags(int fd, int *file_flags, int *and_flags, int *or_flags);
int izo_ioc_get_rcvd(int fd, uuid_t uuid, struct izo_rcvd_rec *rec);
int izo_ioc_adjust_lml(int fd, struct lento_vfs_context *info);
int izo_ioc_get_kml_size(int fd, __u64 *kmlsize);
int izo_ioc_connect(int fd, __u64 ip_address, __u64 port, uuid_t uuid, 
                int client_flag);
int izo_ioc_go_fetch_kml(int fd, uuid_t, __u64 kmlsize);
int izo_ioc_revoke_permit(int fd, uuid_t);
int izo_ioc_set_kmlsize(int fd, __u64 kmlsize, uuid_t uuid);
int izo_ioc_setpid(int fd, __u32 minor);
int izo_ioc_client_make_branch(int fd, char *tagname, char *branchname);
int izo_ioc_server_make_branch(int fd, const char *fsetname);
int izo_ioc_branch_undo(int fd, char *branchname);
int izo_ioc_branch_redo(int fd, char *branchname);
int izo_ioc_set_uid(int fd, int uid);
int izo_ioc_set_channel(int fd, int channel);
int izo_ioc_get_channel(char *cache, int *channel);
int izo_ioc_get_fileid(int fd, char *rec, __u32 size,
                       __u64 *ino, __u64 *generation);
int izo_ioc_set_fileid(int fd, char *path, __u64 ino, __u64 generation);
int izo_ioc_repstatus(int fd, __u64 client_kmlsize,
                      struct izo_rcvd_rec *lr_client,
                      struct izo_rcvd_rec *rec);

/* net.c */
int izo_resolve(char *hostname, __u64 *ip);
int is_socket_new(int nonblock);

/* upcall.c */
struct intersync_upcall { 
        char                     *u_buf;
        struct izo_upcall_hdr    *u_hdr;
        char                     *u_path;
        char                     *u_fset;
        char                     *u_rec;
};

int upcall_unpack(struct intersync_upcall *rec, char **buf, char *end);
char *upcall_print(struct intersync_upcall *rec, int brief);

/* kml_pack.c */
int kml_pack(struct kml_rec *rec, char **buf, int *end);

#ifdef __G_LIB_H__
/* kml_list.c */
GList *kml_listify(char *buf, int len, int first_recno);
GList *kml_listify_file(char *fname, int first_recno);
#endif

/* kml_invert.c */
void kml_invert_free(struct kml_rec *rec);
struct kml_rec *kml_invert(struct kml_rec *rec, int recno);
int kml_invert_closes(void);

/* kml_compare.c */
char *kml_compare(struct kml_rec *a, struct kml_rec *b);

/* util.c */
char * izo_dirname(char * path);
int izo_mkpath(char * dir, int mode);


#endif
